#------------------------------------------------------------------------------
#                                U&R Consultores                               
#                                www.uyr.com.ar 
#------------------------------------------------------------------------------
#                                                                              
# File      : auditawk.awk
#                                                                              
#            Process the output of :
#                                    auditreduce | praudit
#                                                                              
#            and produce an "user-friendly" output.(I hope)
#
# Usage     : 
#   
#            awk -f auditawki.awk [-v ARG_user=username]
#                                 [-v ARG_login=ok/ko]
#                                 [-v ARG_filtro=FilterFile FilterFile]
#                                 FS="," InputFile > OutputFile
#
#                                                                              
#            ARG_user: "username" 
#                      shows Only the activity of this user.
#                                                                              
#            ARG_login: ok -> shows only login (success and failure)
#                              and logout activity.
#
#                       ko -> shows only login failure activity.
#                                                                              
#             ARG_filtro=FilterFile FilterFile
#                        (it is not a mistake twice the same value).
#                        File where there are strings that tag records
#                        to filter out.
#
# Version  : 3.1 - 20000312
#                                                                              
# IMPORTANT: 
#             (1) Use gawk not nawk.
#                                                                              
#             (2) Filtering:
#                 The strings to filter out are searched only
#                 in the following fields:
#                 header, path and exec_args
#                 in the output generated running
#                               auditreduce | praudit
#                                                                 
#                                                                 
#                 Filter file Structure.
#                 Every line has two fields using space as FS.
#                 Blank lines, and lines starting with # are ignored.
#                                                                              
# Creacion : 20000106 - Francisco Mancardi - (FM)                      
#                                                                              
# Revision : 20000312
#
#------------------------------------------------------------------------------

BEGIN { 
       init()
       idx_header=0
       idx_path= 0
       idx_exec_args= 0

       # El archivo de filtro tiene separador de campos
       # distinto al archivo a analizar.
       # Usa el separado standard -> blanco
       FS=" "

}

# *************************************************
# Init filter arrays (if we are using a filter)
# *************************************************
FILENAME == ARG_filtro {

   if( FILENAME !="filtro_nada")
   {

   if($1 == "header") {
         aFheader[idx_header] = $2
         idx_header++
   }
 
   if($1 == "path") {
         aFpath[idx_path] = $2
         idx_path++
   }
 
   if($1 == "exec_args") {
         aFexec_args[idx_exec_args] = $2
         idx_exec_args++
   }

   }
}
# *************************************************

# -------------------------------------------------
# Audit data Processing
# -------------------------------------------------
FILENAME != ARG_filtro {

# ----------------------------------------------------------------------
#               Impresion de cartel de inicio y de fin 
# ----------------------------------------------------------------------
  if( $1 == "file")
  {
    printf "\n%s \n",MARCO 
    if( FNR == 1 )
    {
      # marco que empece a procesar
      printf " Audit Data - Starting date: %s\n", $2 

      delete aheader
      delete asubject 
      delete atext 
      delete areturn 
      delete apath
      delete acmdline 

      if( filtrar_usuario == 1 )
      {
       printf(" *** Showing only Activity  for login name: %s ***\n", ARG_user)
      }


      # ------------------------------------------
      if( solo_login_logout == 1 )
      {
       printf("  *** Showing only login (success/fail) and logout Activity ***\n")
      }
      else
      {
         if( solo_login_logout == 2 )
         {
           printf("  *** Showing only login FAILURE Activity ***\n")
         }
      }
      # ------------------------------------------

      # Print filter rules
      print_filtros()

    }
    else
    {
      printf " Audit Data - Ending date: %s\n", $2 
    }
    printf "%s \n",MARCO 
  }
# ----------------------------------------------------------------------


# ----------------------------------------------------------------------
# Token: Header
# ----------------------------------------------------------------------
  if( $1 == "header")
  {

    # esto marca el inicio (y por lo tanto el fin) de un registro
    # de auditoria
    if( FNR != NR_PRIMER_TOKEN_HEADER)
    {
    # hay que procesar lo que puse en los arrays y borrarlos   

       if( filtrar_usuario == 1 )
       {
         #if( ARG_user == asubject["UID"] )
         #printf("** %s - %s\n", ARG_user, asubject["UID"] )
 
         # por las dudas para comparar el string, primero me fijo
         # que tengan igual longitud.
         # Hago esto pues no se si ana == anabella da como resultado FALSE
         #
         #if( (LEN_ARG_user == length(asubject["UID"])) &&
         #   ARG_user == asubject["UID"] )
         #{

         if( ARG_user == asubject["UID"] )
         {
             dificil(aheader, asubject, atext, areturn, apath, acmdline, hay_cmdline, solo_login_logout, aFpath) 
         }
       }
       else
       { 
        dificil(aheader, asubject, atext, areturn, apath, acmdline, hay_cmdline, solo_login_logout, aFpath) 
       } 


        # *** >> *** >> 
        # init
        path_counter = 0
        text_counter = 0
        hay_cmdline = 0

        # destruyo todos los arrays
        # (solo sirve con gawk)
        delete aheader
        delete asubject 
        delete atext 
        delete areturn 
        delete apath
        delete acmdline 
        # *** >> *** >> 

    }

    # entonces: el anteultimo -> fecha
    #           del 4 a anteultimo-1 -> TEXTO 
    POS_FECHA=(NF-1)
    aheader["FECHA"]=$POS_FECHA

    MY_TEXTO=""
    START_TEXTO=4
    STOP_TEXTO=(POS_FECHA-1)

    # Normalmente en esta posicion esta el string (2)
    # que no aporta en absoluto
    MY_TEXTO=$START_TEXTO
    sub("(2)","",MY_TEXTO)

    for( idx=(START_TEXTO+1); idx < STOP_TEXTO; idx++)
    {
       MY_TEXTO=MY_TEXTO","$idx
    }
    aheader["TEXTO"]=MY_TEXTO

  }
# ----------------------------------------------------------------------

# ----------------------------------------------------------------------
# Token: path
# ----------------------------------------------------------------------
  if( $1 == "path")
  {

    # en ciertos registros puede haber mas de un token path
    path_counter++ 
    #indice = "TEXTO"  
    apath[path_counter]=$2
    
  }
# ----------------------------------------------------------------------

# ----------------------------------------------------------------------
# Token: text
# ----------------------------------------------------------------------
  if( $1 == "text")
  {

    # en ciertos registros puede haber mas de un token text
    text_counter++ 
    atext[text_counter]=$2
    
  }
# ----------------------------------------------------------------------


# ----------------------------------------------------------------------
# Token: subject
# ----------------------------------------------------------------------
  if( $1 == "subject")
  {
    asubject["UID"]=$2
    asubject["EUID"]=$3
    asubject["RUID"]=$5

    # El ultimo campo tiene datos de la maquina desde la cual se 
    # accede.
    # Formato Device-ID Machine-ID (segun la documentacion).
    # de lo observado resulta que hay tres campos separados
    # por blanco y el que me interesa es el ultimo.
    nelem = split($9, myarray, " ")
    asubject["HOST"]=myarray[nelem]
  }
# ----------------------------------------------------------------------


# ----------------------------------------------------------------------
# Token: return
# ----------------------------------------------------------------------
  if( $1 == "return")
  {
    areturn["STATUS"]=$2
  }
# ----------------------------------------------------------------------

# ----------------------------------------------------------------------
# Token: exec_args
# ----------------------------------------------------------------------
  if( $1 == "exec_args" )
  { 
    # leo la proxima linea
    if( getline == STATUS_OK_GETLINE )
    {
      acmdline["LINEA"]=$0
      hay_cmdline = 1
    }
  }
# ----------------------------------------------------------------------


}

END {

}





# ******************************************************************
#                        Funciones Utilitarias
# ******************************************************************

# ------------------------------------------------------------------
#
# Argumentos:
#            aHeader, aSubject, aText, aReturn, aPath, aCmdline 
#            hay_cmdline, solo_login_logout 
#
# Globales : array de Filtros
#
#            aFheader
#            aFpath
#            aFexec_args
#
function dificil(aHeader, aSubject, aText, aReturn, aPath, aCmdline, hay_cmdline, solo_login_logout, aFpath)
{

# ------------------------------------------------------
# Solo se deben mostrar los eventos de login/logout ?
if( solo_login_logout == 1)
{
    # Solo se deben mostrar los eventos de login/logout SIN IMPORTAR
    # su EXITO
    #
    #
    # test por NEGATIVO 
    if ( index(aHeader["TEXTO"],"login") == 0 &&
         index(aHeader["TEXTO"],"logout") ==0)
    {
       # Huida Salvaje >---->
       return  
    }
}
# -----------------------------------------------------

if( solo_login_logout == 2)
{
    # Solo se deben mostrar los eventos de login FALLIDOS
    #
    # test por NEGATIVO 
    if ( index(aHeader["TEXTO"],"login") == 0 )
    {
       # Huida Salvaje >---->
       return  
    }

    # ********************************************
    # es actividad de login 
    if ( index(aHeader["TEXTO"],"login") != 0 )
    {
       if( index(aReturn["STATUS"],"failure") == 0)
       {
          # Huida Salvaje >---->
          return  
       }
    }  # endif
    # ********************************************
}
# ------------------------------------------------------

# ------------------------------------------------------
# 20000212 - FM
# nuevos filtros para eliminar lineas que no aportan
# ------------------------------------------------------
#
# Analisis SOLO sobre las secciones:
# exec_args, header y path
#
# ++++++++++++++++++++++++++++++++++++++++
#for (val in aFpath)
#    printf ("*** Tested %s - \n", aFpath[val])

for (val in aPath)
{
    for ( xval in aFpath )
    {
        #printf (" %s  vs  %s \n", aPath[val], aFpath[xval])

        if( aPath[val] == aFpath[xval])
        {
          # Huida Salvaje >---->
          #printf (" Filtrado %s  vs  %s \n", aPath[val], aFpath[xval])
          return  
        }

    }
}
# ++++++++++++++++++++++++++++++++++++++++


# ++++++++++++++++++++++++++++++++++++++++
for (val in aHeader)
{
    for ( xval in aFheader )
    {
        if( aHeader[val] == aFheader[xval])
        {
          # Huida Salvaje >---->
          #printf (" Filtrado %s  vs  %s \n", aHeader[val], aFheader[xval])
          return  
        }
    }
}
# ++++++++++++++++++++++++++++++++++++++++
if( hay_cmdline == 1)
{
  for (val in aCmdline)
  {
    for ( xval in aFexec_args )
    {
        if( aCmdline[val] == aFexec_args[xval] )
        {
          # Huida Salvaje >---->
          #printf (" Filtrado %s  vs  %s \n", aCmdline[val], aFexec_args[xval])
          return  
        }
    }
  }
} #if hay_..
# ++++++++++++++++++++++++++++++++++++++++
# ------------------------------------------------------


# Ponemos linea indicando la fecha y hora
printf("\n")
#printf("%s\n", aHeader["FECHA"])
printf("%s -> ", aHeader["FECHA"])

# datos de usuario
#
# 20000312
#printf("UID:%s  EUID:%s  RUID:%s - HOST:%s\n", 
#        aSubject["UID"], aSubject["EUID"], aSubject["RUID"], aSubject["HOST"])

printf("UID:%s  EUID:%s  RUID:%s - From IP/machine:%s\n", 
        aSubject["UID"], aSubject["EUID"], aSubject["RUID"], aSubject["HOST"])

# ----------------->>>
# ahora ponemos datos de la operacion realizada
printf("%s %s", aHeader["TEXTO"], GLB_MI_SEP)


# 20000108
for (val in aText)
    printf "%s %s", aText[val], GLB_MI_SEP


for (val in aPath)
    printf "%s %s", aPath[val], GLB_MI_SEP

#printf("\n")
# ----------------->>>


if( hay_cmdline == 1)
{
  #printf("Linea de Comando:%s \n", aCmdline["LINEA"])
  printf("\n")
  printf("cmdline:%s", aCmdline["LINEA"])
}

# Valor de retorno de status
printf("%s %s \n", GLB_MI_SEP, aReturn["STATUS"])



} # fin de la funcion dificil

# Fin del programa
#**************************************************************************** 

# --------------------------------------------------------------
# Funcion para inicializacion de variables
# Son todas globales ??
#
function init()
{

  # FS=","
  process_started = 0
  primer_registro = 1

  dale = 0

  # para manejar mas de un token path en un registro de audit
  path_counter = 0
  filtrar_usuario = 0
  text_counter = 0
  hay_cmdline = 0

  # 20000210 - (cumplea#os de Sofia/Anabella - Anabella/Sofia)
  solo_login_logout = 0


  MARCO=""
  for (idx=1 ; idx <= 78; idx++)
      MARCO=MARCO"*"

#*************************************************************"

  # Es importante cual es el primer token file
  NR_PRIMER_TOKEN_FILE=1

  # Es importante cual es el primer token header
  NR_PRIMER_TOKEN_HEADER=2

  # constante
  STATUS_OK_GETLINE = 1

  #
  GLB_MI_SEP=" + "

  # printf "%s\n\n", ARG_user
  # --------------------------------------------
  # Controlo si me pasaron algo en el argumento
  # ARG_user
  # --------------------------------------------
  LEN_ARG_user = length(ARG_user)
  if( LEN_ARG_user != 0 )
  {
     filtrar_usuario = 1
  }
  else
  {
     filtrar_usuario = 0
  }
  # --------------------------------------------


  # --------------------------------------------
  # Controlo si me pasaron algo en el argumento
  # ARG_login y que me pasaron
  # --------------------------------------------
  LEN_ARG_login = length(ARG_login)
  if( LEN_ARG_login != 0 )
  {
     if( ARG_login == "ok" )
     {
       solo_login_logout = 1
     }
     else
     {
       solo_login_logout = 2
     }
     # printf("%s",ARG_login)
  }
  else
  {
     solo_login_logout = 0
  }
  # --------------------------------------------

  # para manejar los indices dentro de cada array de FILTRO
  idx_path      = 0
  idx_exec_args = 0
  idx_header    = 0
}
# fin de la funcion init()
# --------------------------------------------------------------



# --------------------------------------------------------------
function print_filtros(){

# 20000310 
# Si todos los arrays estan vacios nada escribo
my_counter = 0
for (val in aFpath)
     my_counter++

for (val in aFheader)
     my_counter++

for (val in aFexec_args)
     my_counter++
# ---------------------------------------------

if( my_counter != 0 )
{
        printf ("%s \n",MARCO)
        #printf("Archivo de filtros: %s\n", ARG_filtro)
        printf("Filter File       : %s\n", ARG_filtro)
	printf ("        Rules       \n")
        printf ("%s \n",MARCO)

	for (val in aFpath)
     		printf ("Filter aFpath %s \n", aFpath[val])

	for (val in aFheader)
     		printf ("Filter aFheader %s \n", aFheader[val])

	for (val in aFexec_args)
     		printf ("Filter aFexec_args %s \n", aFexec_args[val])

        printf ("%s \n",MARCO)
	printf ("        Rules      \n")
        printf ("%s \n",MARCO)
}


}
# --------------------------------------------------------------


# Fin del programa
#**************************************************************************** 
#**************************************************************************** 
#**************************************************************************** 
#                    Lineas de ejemplo para archivo de filtro               #
#
# Par usar estas lineas copiarlas y eliminar el caracter de la columna 1
#
## Archivo de filtro para auditawk.awk
## 
#exec_args /usr/sbin/audit,-n
#header    fcntl(2)
#header    fcntl()
#path      /devices/pseudo/mm@0:null
#exec_args /usr/sbin/auditreduce
#exec_args /usr/sbin/praudit
#
#
## -------------------------------------------------------------------
## INTERESANTE APARECE EL CAMBIO DE PASSWORD
## header,88,2,passwd,,Wed Feb 02 10:24:11 2000, + 590004500 msec
## subject,fferreir,root,sys,fferreir,root,18124,18096,24 3 163.35.9.227
## text,update password success
## return,success,0
## -------------------------------------------------------------------
#exec_args /etc/security/newauditlog.sh
#exec_args ls
#path /var/mail/
#exec_args /usr/sbin/quota
#exec_args /bin/cat,-s,/etc/motd
#
